
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% F. Amato, C. Cosentino,  F. Montefusco
% Biomechatronics Laboratory
% Universita' Magna Graecia di Catanzaro
% Campus Universitario "Salvatore Venuta"
% viale Europa, loc. Germaneto
% 88100 Catanzaro ITALY
% Tel:+39-0961-369-4196
% fax: +39-0961-369-4090
% e-mail: f.montefusco@unicz.it
% 
% This script allows to generate in silico networks, represented as
% dynamical linear system in the form dot(x)= Ax +Bu, whose A matrices 
% determines both the dynamics and the topology of the network. In 
% particular, the networks have a random Erdos_Renyi topology.
% To this aim, the following functions are used:
% sprand.m, gen_ss_scale_free.m, sel_constr.m, gen_data.m, smth_int_data.
% The main parameters (as the density), that determine the 
% properties of the in silico networks, can be changed to edit in the 
% beginning of this file.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% DEFINE A SET OF PARAMETERS TO GENERATE THE NETWORKS %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
clc
close all

% define a part of the filename where your networks data are saved, for example:
% str_filename='in_silico_rand'
str_filename=input('digit part of the filename, where your networks are saved: ','s');

% number of nodes of the graph/network, we set to 10 and 20
n_nodes=10;

% number of generated networks
num_networks=4;

% eig_val_inf: input variable of the function gen_ss_scale_free.m, we
% generally set it to -2,
eig_val_inf=-2;

% eig_val_sup: input variable of the function gen_ss_scale_free.m, we set
% it to -0.5
eig_val_sup=-0.5;

% pk:fraction of the constraints to be imposed
pk=0.3;

% fraction of the nonzero coefficients of the network
density=0.26; 
    
for net=1:num_networks
    
    % generate a biological in silico network with a random distribution
    % the sparsisty coefficient is in the interval
    % [spar_coef_min spar_coef_max]; check the stability
    spar_coef=1;   
    spar_coef_min=0.65; % minimum value of the sparsisty coefficent
    spar_coef_max=0.85; % maximum value of the sparsisty coefficent
    unstable_sys=true; % flag true when the system is unstable 
    num_it=1; % iteration counter
    while spar_coef<=spar_coef_min || spar_coef>=spar_coef_max || unstable_sys 
        if num_it>20
            error('warning spar_coef, modify the spar_coef range, increase/ decrease n_link or prob_p')
        end
        
        % generation of the network topology with random distribution
        Cnct_Mtx=sign(abs(full(sprand(n_nodes,n_nodes,density))+diag(rand(n_nodes,1))));
      
        % generation of a stable dynamical linear system in form
        % dot(x)=Ax+Bu
        % whose A matrix has a structure corresponding to the desired topology
        % determined by the connectivity matrix Cnct_Mtx
        [original_network,B_vct,unstable_sys]=gen_ss_scale_free(Cnct_Mtx,eig_val_inf,eig_val_sup);
        n_elem_not_null=nnz(original_network);
        spar_coef=(n_nodes^2-n_elem_not_null)/(n_nodes^2-n_nodes);
        num_it=1+num_it;
    end

    % generation of the constraints matrix
    Constr_mtx_in= sel_constr(original_network,pk);


    % number of time points of the data set to genenerate
    time_points=20;


    plot_data=true; % flag true when we want to plot the data set,
                    % if you select a number of generated networks major
                    % than of five, it is preferible set this parameter to
                    % false
    % calculate the state response of the generated biological in silico
    % network
    [Meas_mtx_20tp Meas_mtx_20tp_noise10 Meas_mtx_20tp_noise30]=gen_data(original_network,B_vct,time_points,plot_data);

    % filter the measurement noise and increase by interpolation
    % the number of observations.
    sample_dt=3;% increase the number of points in the dt interval
    Meas_mtx_20tp_noise10_smth=smth_int_data(Meas_mtx_20tp_noise10,sample_dt,plot_data);
    Meas_mtx_20tp_noise30_smth=smth_int_data(Meas_mtx_20tp_noise30,sample_dt,plot_data);
    
    % if you want a different level of noise, you can run the function
    % gen_data_noise
    %     [Meas_mtx Meas_mtx_noise]=gen_data_noise(original_network,...
    %         B_vct,time_points,perc_noise,plot_data,x0);
    
    clear spar_coef_min spar_coef_max unstable_sys plot_data n_elem_not_null num_it
    
    eval(['save ',str_filename,'_',num2str(n_nodes),'n_num_',num2str(net)]); 
end    
